<?php

require_once dirname(dirname(__FILE__)) . '/bootstrap/start.php';

class CambioReal_Pay_Model_Payment extends Mage_Payment_Model_Method_Abstract {

	protected $_code = 'cambioreal';

	protected $_isGateway          = true;
	protected $_isInitializeNeeded = false;

	/**
	* Allowed operations
	*/
	protected $_canAuthorize           = true;
	protected $_canCapture             = true;
	protected $_canCapturePartial      = false;
	protected $_canRefund              = false;
	protected $_canVoid                = true;
	protected $_canCancel              = true;
	protected $_canUseInternal         = false;
	protected $_canUseCheckout         = true;
	protected $_canUseForMultishipping = false;

	/**
	* The form block
	*
	* @var string
	*/
	protected $_formBlockType = 'cambioreal/form';

	/**
	* Gets the payment controller checkout URL
	*
	* @return string
	*/
	public function getOrderPlaceRedirectUrl()
	{
		Mage::log('Redirecting to ' . $_SESSION['crRedirectUrl']);
		return $_SESSION['crRedirectUrl'];
	}

	/**
	* Voids an order
	*
	* @param  Varien_Object $payment
	* @return Mage_Payment_Model_Method_Abstract
	*/
	public function void(Varien_Object $payment)
	{
		parent::void($payment);

		$cr_token = $payment->getData('cr_token');
		Mage::log('Void order ' . $cr_token);

		$response = \CambioReal\CambioReal::cancel(array(
			'token' => $cr_token
		));

		return $this;
	}

	/**
	* Cancels an order
	*
	* @param  Varien_Object $payment
	* @return Mage_Payment_Model_Method_Abstract
	*/
	public function cancel(Varien_Object $payment)
	{
		$cr_token = $payment->getData('cr_token');
		Mage::log('Cancel order ' . $cr_token);

		return $this->void($payment);
	}

	/**
	* Authorizes a transaction
	*
	* @param  Varien_Object $payment
	* @param  float $amount
	* @return Mage_Payment_Model_Method_Abstract
	*/
	public function authorize(Varien_Object $payment, $amount)
	{
		parent::authorize($payment, $amount);

		// $session = Mage::getSingleton('checkout/session');
		$order = $payment->getOrder();
		$currencyCode = $order->getBaseCurrencyCode();

		Mage::log('Authorizing order [' . $order->getIncrementId() . ']');

		// Street number workaround
		$streetNumber = preg_replace('/\D/', '', $order->getBillingAddress()->getData('street'));
		$streetNumber = ($streetNumber > 0) ? $streetNumber : '1';

		$amount_total = $order->getBaseGrandTotal();

		if ($currencyCode === 'USD' && $amount_total > 2999)
		{
			Mage::throwException(Mage::helper('cambioreal')->__('For this method of payment is allowed only values below %1$s %2$s.', 3000, 'USD'));
		}

		if ($currencyCode === 'BRL' && $amount_total > 10000)
		{
			Mage::throwException(Mage::helper('cambioreal')->__('For this method of payment is allowed only values below %1$s %2$s.', 10000, 'BRL'));
		}

		$due_date  = Mage::getStoreConfig('payment/cambioreal/due_date');
		$duplicate = Mage::getStoreConfig('payment/cambioreal/duplicate');

		// On guest checkout, get billing email address
		$email = $order->getCustomerEmail() ?: $order->getBillingAddress()->getEmail();

		$products_result[] = array(
			'descricao'  => Mage::app()->getStore()->getFrontendName() . ' order',
			'base_value' => $amount_total,
			'valor'      => $amount_total,
			'qty'        => 1,
    	);

		$data = array(
			'client' => array(
				'name'  => $order->getCustomerFirstname() . ' ' . $order->getCustomerLastname(),
				'email' => $email,
			),
			'currency'  => $currencyCode,
			'amount'    => $amount_total,
			'order_id'  => $order->getIncrementId(),
			'duplicate' => $duplicate,
			'due_date'  => $duplicate == 0 && empty($due_date) ? null : $due_date,
			'products'  => $products_result,
		);

		try
		{
			Mage::log('Authorizing order [' . $order->getIncrementId() . '] - calling CambioReal');

			$response = \CambioReal\CambioReal::request($data);

			if (empty($response)) {
				Mage::log('Authorizing order [' . $order->getIncrementId() . '] - error: unknown.');
				Mage::throwException('Unknown error.');
			}

			if ($response->status !== 'success')
			{
				Mage::log('Authorizing order [' . $order->getIncrementId() . '] - error: ' . $response->message);
				Mage::throwException($response->message);
			}

			Mage::log('Authorizing order [' . $order->getIncrementId() . '] - CambioReal authorized!');

			$cr_token = $response->data->token;
			$cr_code = $response->data->code;

			// Add the CambioReal hash in the order data
			$order->getPayment()
				  ->setData('cr_token', $cr_token)
				  ->save();

            // Add to status history
            $order->addStatusToHistory($orderStatus, "Payment request created in CambioReal ($cr_code).", false)
                  ->save();

            // Return URL
            $successUrl = Mage::getUrl('checkout/onepage/success') . '?token=' . $cr_token;
            $failureUrl = Mage::getUrl('checkout/onepage/failure');

            if (isset($response->data->checkout))
            {
                // Redirect to Checkout CambioReal site
                $params = http_build_query(array(
                    'url_callback' => $successUrl,
                    'url_error'    => $failureUrl,
                ));

                $successUrl = $response->data->checkout;

                if (strpos($successUrl, '?') === false)
                {
                    $successUrl .= '?';
                }

                $successUrl .= $params;
            }

            // Redirect to CambioReal success page on client store
			$_SESSION['crRedirectUrl'] = $successUrl;
		}
		catch (Exception $e)
		{
			Mage::throwException($e->getMessage());
		}

		return $this;
	}

}
