<?php

class CambioReal_Pay_PaymentController extends Mage_Core_Controller_Front_Action {

	protected function _construct()
	{
		require_once dirname(dirname(__FILE__)) . '/bootstrap/start.php';
	}

    /**
     * Notification URI for the CambioReal status changes
     *
     * @return void
     */
    public function notifyAction()
    {
        $cr_token = $this->getRequest()->getParam('token');

        if ( ! $cr_token)
        {
            echo 'Error: empty request.';
            return;
        }

        $orderPayment = Mage::getModel('sales/order_payment')
                            ->getCollection()
                            ->addFieldToFilter('cr_token', $cr_token)
                            ->getFirstItem();

        if ( ! $orderPayment->getParentId())
        {
            echo 'Error: order_payment not found.';
            return false;
        }

        $response = \CambioReal\CambioReal::get(array('token' => $cr_token));

        if ($response->status == 'success')
        {
            try
            {
                // Get the new status from Magento
                $orderStatus = $this->_getOrderStatus($response->data->status);

                // Update order status
                $order = Mage::getModel('sales/order')
                            ->load($orderPayment->getParentId(), 'entity_id');

                // Checks if the order exists
                if ( ! $order->getRealOrderId())
                {
                    throw new Exception('Order cannot be found.');
                }

                Mage::log('Notify Order [' . $order->getRealOrderId() . '] - status: ' . $response->data->status);

                // If payment status is Canceled or payment status is BOLETO_EXPIRADO and duplicate is not activated, order can be cancelled
                if (in_array($response->data->status, $this->_getCanceledStatus()) ||
                    ($response->data->status == 'BOLETO_EXPIRADO' &&
                        ($response->data->duplicate == 0 || $response->data->expired)
                    )
                )
                {
                    if ( ! $order->canCancel())
                    {
                        throw new Exception('Order cannot be canceled, assuming already processed.');
                    }

                    // Cancel order
                    $order->cancel();

                    // Set orderStatus to Generic canceled status - nothing more to do
                    $orderStatus = 'canceled';

                    // Comment on order
                    $order->addStatusHistoryComment('Automatically CANCELED by CambioReal Notification System.', false);
                }

                // If payment status is SOLICITACAO_FINALIZADA - Paid - AND order can be invoiced
                elseif ($response->data->status == 'SOLICITACAO_FINALIZADA')
                {
                    // If can NOT Invoice or order is not new
                    if ( ! $order->canInvoice())
                    {
                        throw new Exception('Order cannot be invoiced, assuming already processed.');
                    }

                    // Invoice
                    $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
                    $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);
                    $invoice->register();
                    $invoice->getOrder()->setCustomerNoteNotify(true);
                    $invoice->getOrder()->setIsInProcess(true);

                    // Commit invoice to order
                    $transactionSave = Mage::getModel('core/resource_transaction')
                              ->addObject($invoice)
                              ->addObject($invoice->getOrder());
                    $transactionSave->save();

                    // Comment on order
                    $order->addStatusHistoryComment('Automatically INVOICED by CambioReal Notification System.', false);
                    $order->getSendConfirmation();
                    $order->setEmailSent(true);
                    $order->sendNewOrderEmail();
                }

                // Add to status history
                $order->addStatusToHistory($orderStatus, 'Status change notified by CambioReal Notification System.', false)
                      ->save();

                echo 'OK: payment ' . $cr_token . ' was updated<br>status: '.$response->data->status;
            }
            catch (Exception $e)
            {
                echo 'Error: payment ' . $cr_token . ' could not update order, Exception: '  . $e->getMessage() . '<br>';
            }
        }
        else
        {
            echo 'Error: payment ' . $cr_token . ' could not be updated.<br>';
        }
    }

    /**
     * Get the new order status from Magento
     *
     * @param  string $status The CambioReal order status code
     * @return string The Magento order status
     */
    protected function _getOrderStatus($status)
    {
        $orderStatus = array(
            'PENDENTE_ACCOUNT'       => Mage::getStoreConfig('payment/cambioreal/order_status'),
            'AGUARDANDO_CLIENTE'     => Mage::getStoreConfig('payment/cambioreal/order_status'),
            'BOLETO_GERADO'          => Mage::getStoreConfig('payment/cambioreal/order_status'),
            'BOLETO_EXPIRADO'        => Mage::getStoreConfig('payment/cambioreal/order_status'),
            'BOLETO_CANCELADO'       => Mage::getStoreConfig('payment/cambioreal/canceled_order_status'),
            'SOLICITACAO_RECUSADA'   => Mage::getStoreConfig('payment/cambioreal/canceled_order_status'),
            'SOLICITACAO_INVALIDA'   => Mage::getStoreConfig('payment/cambioreal/canceled_order_status'),
            'SOLICITACAO_CANCELADA'  => Mage::getStoreConfig('payment/cambioreal/canceled_order_status'),
            'SOLICITACAO_PAGO'       => Mage::getStoreConfig('payment/cambioreal/order_status'),
            'SOLICITACAO_FINALIZADA' => Mage::getStoreConfig('payment/cambioreal/paid_order_status'),
            'SOLICITACAO_EXPIRADA'   => Mage::getStoreConfig('payment/cambioreal/canceled_order_status'),
        );

        return $orderStatus[$status];
    }

    protected function _getCanceledStatus()
    {
        return array(
            'BOLETO_CANCELADO',
            'SOLICITACAO_RECUSADA',
            'SOLICITACAO_INVALIDA',
            'SOLICITACAO_CANCELADA',
            'SOLICITACAO_EXPIRADA',
        );
    }

}
